package concurrentcalc;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.concurrent.Executor;



public class Setup {

	public static void main(String[] args) {
		
		// Konstante legen den Wertebereich fest
		final int maxRange = 10000;
		final int segments = 10;
		final int segmentSize=maxRange / segments;
		
		final ArrayList <FindPrimeNumbers> results = new ArrayList<FindPrimeNumbers>();
		
		// Berechnen der Primzahlen in einzelnen Segmenten
		int startIndex = 0;
		
		Executor executor;
		String type = args.length>0 ? args[0] : "pool";

		if (type.equals("single")) {
			executor = new SingleThreadExecutor();
		} else if (type.equals("multi")) {
			executor = new MultiThreadExecutor();
		} else {
			executor = new ThreadPoolExecutor();
		}		
		
		System.out.println("Executor type:" + type);
		
		long start = System.nanoTime();
			
		while (startIndex < maxRange) {
			// Teilabschnitt festlegen
			final int endIndex = (startIndex + segmentSize < maxRange) ? startIndex + segmentSize : maxRange;

			// FindPrimeNumbers mit Wertebereich intitialisieren,
			// Primzahlen finden und ausgegeben
			final FindPrimeNumbers pn = new FindPrimeNumbers (startIndex,endIndex);
		
	
			
			executor.execute(pn);
			results.add(pn);	// keine direkte Ausgabe mehr, sondern Liste fr sptere Ergebnisabfrage	
			
			// nchstes Teilsegment in while-Schleife durchlaufen
			startIndex = endIndex;
			
		} // end while
		
		

		// Nachdem alle Berechnungen gestartet sind, knnen wir die Ergebnisse
		// der Reihe nach ausgeben sobald diese vorliegen.
		// waitForResult blockiert diesen Thread solange das Ergebnis noch nicht berechnet ist.
		Iterator<FindPrimeNumbers> i = results.iterator(); 
		while (i.hasNext()) {
			String s = i.next().waitForResult().toString();
			System.out.println(s);
		}
		
		long end = System.nanoTime();
		long time = end - start;
		
		System.out.println ("Calculation took " + time + " nanoseconds.");	


	}	
	
}
